from typing import Union
import pandas as pd
import datetime as dt
from itertools import groupby
import json
import SeisMonitor.utils as sut
from obspy import read_inventory
from obspy.core.event.catalog import Catalog, read_events
from obspy.core.inventory.inventory import (Inventory,read_inventory)

def changing_picks_info(ev,ref_picks):
    true_picks = []
    picks_dict = {}
    for pick in ev.picks:
        station = pick.waveform_id.station_code
        phasehint = pick.phase_hint
        try:
            time = pick.time.strftime("%Y%m%dT%H%M%S")
            true_pick = ref_picks[station+"_"+phasehint+"_"+time]
        except:
            try:
                time = (pick.time-dt.timedelta(seconds=1)).strftime("%Y%m%dT%H%M%S")
                true_pick = ref_picks[station+"_"+phasehint+"_"+time]
            except:
                try:
                    time = (pick.time+dt.timedelta(seconds=1)).strftime("%Y%m%dT%H%M%S")
                    true_pick = ref_picks[station+"_"+phasehint+"_"+time]
                except:
                    sut.printlog("warning","NLLOC",f'no information could be extracted from the mext pick in the input catalog: {station+"_"+phasehint+"_"+time}')

        # print(list(ref_picks.keys()))

        # picks_dict[pick.resource_id.id] = true_pick.resource_id.id
        picks_dict[pick.resource_id.id] = true_pick
        true_picks.append(true_pick)

    ori_pref  = ev.preferred_origin()
    true_arrivals = []
    for arrival in ori_pref.arrivals:
        try:
            arrival_prob = json.loads(picks_dict[arrival.pick_id.id].comments[0].text)
            arrival.time_weight = arrival_prob["probability"]
        except:
            print("picks don't have probability")
        arrival.comments = picks_dict[arrival.pick_id.id].comments
        # print(picks_dict[arrival.pick_id.id])
        arrival.pick_id.id = picks_dict[arrival.pick_id.id].resource_id.id
        true_arrivals.append(arrival)

    

    ori_pref.arrivals = true_arrivals
    ev.picks = true_picks
    # for arrival in ori_pref.arrivals:
    return ev

def get_picks(catalog):
    picks = {}
    for ev in catalog:
        for pick in ev.picks:
            station = pick.waveform_id.station_code
            phasehint = pick.phase_hint
            time = pick.time.strftime("%Y%m%dT%H%M%S")
            picks[station+"_"+phasehint+"_"+time] = pick
    return picks

def get_bad_and_good_events(reloc_catalog):

    def all_equal(iterable):
        g = groupby(iterable)
        return next(g, True) and not next(g, False)
    bad_events = []
    good_events = []
    for ev in reloc_catalog:
        pref_origin = ev.preferred_origin()
        arrivals = pref_origin.arrivals
        azimuths = [x.azimuth for x in arrivals] 
        equal = all_equal(azimuths)
        if equal:
            bad_events.append(ev)
                
        else:
            good_events.append(ev)
    return good_events,bad_events

def filter_arrivals_by_distance(events,distance,
                                        min_P_phases=3,
                                        min_S_phases=2):
    new_events = []
    for ev in events:
        pref_origin = ev.preferred_origin()
        arrivals = pref_origin.arrivals

        picks = {}
        for pick in ev.picks:
            picks[pick.resource_id.id] = pick

        new_arrivals = []
        counts = {"P":[],"S":[]}
        for arrival in arrivals:
            if arrival.distance <= distance:
                new_arrivals.append(arrival)
            else:
                picks.pop(arrival.pick_id.id, None)

            counts[arrival.phase].append(arrival.phase)
        
        counts["P"] = len(counts["P"])
        counts["S"] = len(counts["S"])

        if (counts["P"] < min_P_phases) or\
            (counts["S"] < min_S_phases) :
            continue

        for i,origin in enumerate(ev.origins):
            if origin.resource_id.id == ev.preferred_origin_id:
                ev.origins[i].arrivals = new_arrivals 
            else:
                continue

        ev.picks = list(picks.values())
        new_events.append(ev)
    return new_events

def resp2df(resp):
    """
    Parameters:
    -----------
    resp: str
        RESP filepath

    Returns: DataFrame
        Dataframe with the next columns
        network,station,latitude,longitude,elevation
    """
    networks = []
    stations = []
    longitudes = []
    latitudes = []
    elevations = []

    if isinstance(resp,str):
        inv = read_inventory(resp)
    else:
        inv = resp

    for net in inv:
        for sta in net:
            latitudes.append(sta.latitude)
            longitudes.append(sta.longitude)
            elevations.append(sta.elevation)
            stations.append(sta.code)
            networks.append(net.code)

    df = {"network":networks,"station":stations,
        "latitude":latitudes,"longitude":longitudes,
        "elevation":elevations}
    df = pd.DataFrame(df)
    return df 

class VelModel():
    def __init__(self,vel_path,
                model_name=None,
                vp_vs_ratio=1.78,
                compute_vs=True):
        self.model_name = model_name
        self.vel_path = vel_path
        self.vp_vs_ratio = vp_vs_ratio
        self.compute_vs = compute_vs
        self.vel = pd.read_csv(vel_path)

    def to_nlloc(self,out):
        vm = open(out, 'w')
        msg ="# model layers (LAYER depth, Vp_top, Vp_grad, Vs_top, Vs_grad, p_top, p_grad)\n"
        vm.write(msg)
        for i,row in self.vel.iterrows():
            if self.compute_vs:
                vs = row.vp/self.vp_vs_ratio
            else:
                vs = row.vs

            if i == len(self.vel)-1:
                enter = ""
            else:
                enter = "\n"

            msg = f"LAYER    {row.depth:<6.2f}    {row.vp:<.2f}    0.00    {vs:<.2f}    0.00    {row.rho:<.2f}    0.00{enter}"
            vm.write(msg)
        vm.close()

class Stations():
    def __init__(self,stations) -> None:
        self.stations = resp2df(stations)
        # else:
        #     raise Exception("Error: NLLOC Stations class")

    def to_nlloc(self,out):
        vs = open(out, 'w')
        msg = "# GTSRCE label LATLON latSrce longSrce zSrce elev\n"
        vs.write(msg)
        for i,row in self.stations.iterrows():
            elv = row.elevation/1e3
            if i == len(self.stations)-1:
                enter = ""
            else:
                enter = "\n"

            msg = f"GTSRCE  {row.station:<5}  LATLON  {row.latitude:<7.3f}  {row.longitude:<8.3f}  0.000  {elv:<7.3f}{enter}"
            vs.write(msg)
        vs.close()
        
class LocatorBasicInputs():
    def __init__(self,
                vel_model:VelModel,
                stations:Stations):
        self.vel_model = vel_model
        self.stations = stations

        



import numpy as np
import pandas as pd
import json
import math
import datetime as dt
import pyproj
import os
from . import utils as ut
from SeisMonitor.utils import isfile
from gamma.utils import association
from obspy.core.inventory.inventory import (Inventory,read_inventory)
from tqdm import tqdm
from obspy import UTCDateTime
from obspy.core.event.event import Event
from obspy.core.event import ResourceIdentifier,Catalog
from obspy.core.event.base import CreationInfo
from obspy.core.event.origin import Pick
from obspy.core.event.base import (QuantityError,
                                WaveformStreamID,
                                CreationInfo,
                                Comment)
from obspy.core.event import ResourceIdentifier
from obspy.core.event.origin import Origin, OriginQuality
from obspy.core.event.base import QuantityError
from obspy.core.event.origin import Arrival

class GaMMAObj():
    def __init__(self,region,epsg_proj,
                use_dbscan=True,use_amplitude=True,
                dbscan_eps=10.0,dbscan_min_samples=3,
                vel = {"p": 7.0, "s": 7.0 / 1.75},
                method="BGMM",oversample_factor=20,
                min_picks_per_eq=5,max_sigma11=2.0,
                max_sigma22=1.0,max_sigma12=1.0,
                calculate_amp=True,p_window=10,
                s_window=5,waterlevel=10
                ):
        self.lon_lims = region[0:2]
        self.lat_lims = region[2:4]
        self.z_lims = region[4:]
        self.epsg_proj = epsg_proj
        self.use_dbscan = use_dbscan
        self.use_amplitude = use_amplitude
        self.vel = vel
        self.dbscan_eps = dbscan_eps
        self.dbscan_min_samples = dbscan_min_samples
        self.method = method
        self.oversample_factor = oversample_factor
        self.min_picks_per_eq = min_picks_per_eq
        self.max_sigma11 = max_sigma11
        self.max_sigma22 = max_sigma22
        self.max_sigma12 = max_sigma12
        self.dims = ["x(km)", "y(km)", "z(km)"]
        self.calculate_amp=calculate_amp
        self.p_window=p_window
        self.s_window=s_window
        self.waterlevel=waterlevel

        self.config = self._get_config()
        self.response = None
        self.name = "GaMMA"

    # def _get_config(self):

    #     config = self.__dict__
    #     config["degree2km"] = 111.195
    #     config["center"] = [np.mean(self.lon_lims), np.mean(self.lat_lims)]
    #     config["x(km)"] = ((np.array(self.lon_lims) - config["center"][0]) * config["degree2km"]).tolist()
    #     config["y(km)"] = ((np.array(self.lat_lims) - config["center"][1]) * config["degree2km"]).tolist()
    #     config["z(km)"] = np.array(self.z_lims).tolist()
    #     config["bfgs_bounds"] = [list(config[x]) for x in config["dims"]] + [[None, None]]
    #     return config

    # @property
    # def stations(self):
    #     stations = ut.get_stations_GaMMA_df(self.response)
    #     config = self._get_config()
    #     stations["x(km)"] = stations["longitude"].apply(lambda x: (x - config["center"][0]) * config["degree2km"])
    #     stations["y(km)"] = stations["latitude"].apply(lambda x: (x - config["center"][1]) * config["degree2km"])
    #     stations["z(km)"] = stations["elevation(m)"].apply(lambda x: -x / 1e3)
    #     return stations
        
    def add_response(self,response):
        if isinstance(response,Inventory):
            self.response = response
        else:
            self.response = read_inventory(response)

    def _get_config(self):

        config = self.__dict__

        in_proj = pyproj.Proj("EPSG:4326")
        out_proj = pyproj.Proj(self.epsg_proj)
        y_min,x_min = pyproj.transform(in_proj,out_proj,self.lat_lims[0],self.lon_lims[0])
        y_max,x_max = pyproj.transform(in_proj,out_proj,self.lat_lims[1],self.lon_lims[1])


        config["x(km)"] = np.array([x_min,x_max])/1e3
        config["y(km)"] = np.array([y_min,y_max])/1e3
        config["z(km)"] = np.array(self.z_lims)
        config["bfgs_bounds"] = (
                                (config["x(km)"][0] - 1, config["x(km)"][1] + 1),  # x
                                (config["y(km)"][0] - 1, config["y(km)"][1] + 1),  # y
                                (0, config["z(km)"][1] + 1),  # x
                                (None, None),  # t
                            )
        return config

    @property
    def stations(self):

        if self.response == None:
            raise Exception("You must add response file to the GaMMA Object")

        stations = ut.get_stations_GaMMA_df(self.response)

        in_proj = pyproj.Proj("EPSG:4326")
        out_proj = pyproj.Proj(self.epsg_proj)

        stations["y(km)"] = stations.apply(lambda x: pyproj.transform(in_proj,out_proj,
                                            x["latitude"], x["longitude"])[0] / 1e3, axis=1)
        stations["x(km)"] = stations.apply(lambda x: pyproj.transform(in_proj,out_proj,
                                            x["latitude"], x["longitude"])[1] / 1e3, axis=1)
        stations["z(km)"] = stations["elevation(m)"] / -1e3

        stations = stations.drop_duplicates(ignore_index=True)
        return stations

def get_gamma_picks(event_picks):
    pick_list = []
    for i, row in event_picks.iterrows():
        # print(row.location)
        if math.isnan(row.location):
            loc = ""
        else:
            loc = "{:02d}".format(int(row.location))

        str_id = ".".join((row.network,row.station,
                           loc,row.instrument_type + "Z"))
        
        comment = {'probability': row.prob,
                'GaMMA_probability':row.prob_gamma}

        if row.author == "EQTransformer":
            comment['snr'] = row.snr
            comment['detection_probability'] = row.detection_probability
            comment['event_start_time'] = row.event_start_time.strftime("%Y-%m-%d %H:%M:%S.%f")
            comment['event_end_time'] = row.event_end_time.strftime("%Y-%m-%d %H:%M:%S.%f")
       
        pick_obj = Pick(resource_id=ResourceIdentifier( id= row.pick_id,prefix="pick"),
                                time=UTCDateTime(row.timestamp),
                                time_errors=QuantityError(uncertainty=20/100, # 20 samples before and 20 samples after
                                                        confidence_level=row.prob*100),
                                waveform_id=WaveformStreamID(network_code=row.network,
                                                             station_code=row.station,
                                                             location_code=loc,
                                                             channel_code=row.instrument_type + "Z",
                                                             resource_uri= ResourceIdentifier(id= str_id ),
                                                             seed_string = str_id),
                                phase_hint = row["type"],
                                evaluation_mode = 'automatic',
                                creation_info= CreationInfo(author=row.author,
                                                            creation_time=UTCDateTime.now()),
                                method_id=row.author,
                                comments= [Comment(text=json.dumps(comment))]
                                )
        pick_list.append(pick_obj)
    
    return pick_list

def picks2arrivals(picks):
    arrivals = []
    for pick in picks:
        arrival = Arrival( resource_id =ResourceIdentifier(id=pick.resource_id.id,
                                                            prefix='arrival'),
                           pick_id = pick.resource_id,
                           phase = pick.phase_hint,
                           time_weight = pick.time_errors.confidence_level/100,
                           creation_info=pick.creation_info )
        arrivals.append(arrival)
    return arrivals
        
def get_gamma_origin(catalog_info,event_picks,in_proj="EPSG:3116",
                    out_proj ="EPSG:4326"):
    y,x = catalog_info["y(km)"]*1e3,catalog_info["x(km)"]*1e3
    in_proj = pyproj.Proj(in_proj) #colombia/MAGNA SIRGAS BOGOTÁ/ZONE
    out_proj = pyproj.Proj(out_proj)
    lat,lon = pyproj.transform(in_proj,out_proj,y,x)

    origin = Origin(resource_id = ResourceIdentifier(id=UTCDateTime(catalog_info.time).strftime("%Y%m%d.%H%M%S.%f"),
                                                    prefix='origin'),
                    time = UTCDateTime(catalog_info.time),
                    time_errors = QuantityError(uncertainty=catalog_info.sigma_time),
                    longitude = lon,
                    longitude_errors = QuantityError(),
                    latitude = lat,
                    latitude_errors = QuantityError(),
                    depth = catalog_info["z(km)"],
                    # depth = catalog_info["z(km)"]*1e3,
                    depth_errors = QuantityError(),
                    method_id = ResourceIdentifier(id="GaMMA"),
                    arrivals = picks2arrivals(event_picks),
                    quality = OriginQuality(associated_phase_count=len(event_picks)),
                    evaluation_status = "preliminary",
                    evaluation_mode = "automatic",
                    creation_info = CreationInfo(author="SeisMonitor",
                                                creation_time=UTCDateTime.now())
                     )
    return origin

def get_gamma_catalog(picks_df,catalog_df):

    events = []
    for i, row in catalog_df.iterrows():
        picks = picks_df[picks_df["event_idx"]==i]
        picks = get_gamma_picks(picks)
        origin = get_gamma_origin(row,picks)
        ev = Event(resource_id = ResourceIdentifier(id=origin.resource_id.id,
                                                            prefix='event'),
                    event_type = "earthquake",
                    event_type_certainty = "known",
                    picks = picks,
                    amplitudes = [],
                    focal_mechanisms = [],
                    origins = [origin],
                    magnitudes = [],
                    station_magnitudes = [],
                    creation_info = CreationInfo(author="SeisMonitor",
                                                creation_time=UTCDateTime.now())
                    )
        ev.preferred_origin_id = origin.resource_id.id
        events.append(ev)
    
    catalog = Catalog(events = events,
                    resource_id = ResourceIdentifier(prefix='catalog'),
                    creation_info = CreationInfo(author="SeisMonitor",
                                    creation_time=UTCDateTime.now()) )
    return catalog


class GaMMA():
    def __init__(self,gamma_obj):
        self.gamma_obj = gamma_obj

    def associate(self,picks_csv,xml_path,out_dir):
        self.picks_csv = picks_csv
        self.xml_path = xml_path
        self.response = read_inventory(xml_path)
        self.out_dir = out_dir
        self.xml_out_file = os.path.join(out_dir,"associations.xml")
        self.catalog_out_file = os.path.join(out_dir,"catalog.csv")
        self.picks_out_file = os.path.join(out_dir,"picks.csv")

        picks_df = ut.get_picks_GaMMa_df(self.picks_csv,
                                self.response,
                                compute_amplitudes=self.gamma_obj.calculate_amp,
                                p_window=self.gamma_obj.p_window,
                                s_window=self.gamma_obj.s_window,
                                waterlevel = self.gamma_obj.waterlevel)
        stations = list(set(picks_df["station"].to_list()))

        self.gamma_obj.add_response(self.response)
        station_df = self.gamma_obj.stations
        station_df =  station_df[station_df["station_name"].isin(stations)]
        station_df = station_df.reset_index(drop=True)

        config = self.gamma_obj.__dict__
        pbar = tqdm(1)
        meta = station_df.merge(picks_df["id"], how="right", on="id")

        # print(picks_df, station_df)
        catalogs, assignments = association(picks_df, station_df, config, 
                                method=self.gamma_obj.method,pbar=pbar)
        # print(catalogs, assignments,config)
        if not catalogs:
            return Catalog(),pd.DataFrame(),pd.DataFrame()
        
        catalog = pd.DataFrame(catalogs)
        
        catalog["time"] = pd.to_datetime(catalog["time"],format="%Y-%m-%dT%H:%M:%S.%f")
        catalog["event_index"] = catalog["time"].apply(lambda x: x)

        assignments = pd.DataFrame(assignments,
                                columns=["pick_index", "event_idx", "prob_gamma"])
        assignments = assignments.set_index("pick_index")
        

        picks = pd.merge(picks_df, assignments, left_index=True, 
                        right_index=True, how='right')
        isfile(self.catalog_out_file)
        catalog.to_csv(self.catalog_out_file,index=False)
        isfile(self.picks_out_file)
        picks.to_csv(self.picks_out_file,index=False)
       
        obspy_catalog = get_gamma_catalog(picks,catalog)
        isfile(self.xml_out_file)
        obspy_catalog.write(self.xml_out_file,format="SC3ML")


        return obspy_catalog,catalog,picks

if __name__ == "__main__":
    csv = "/home/emmanuel/EDCT/SeisMonitor/SeisMonitor/monitor/associator/test.csv"
    cat_csv = "/home/emmanuel/EDCT/SeisMonitor/SeisMonitor/monitor/associator/cat_test.csv"
    df = pd.read_csv(csv)
    df["timestamp"] = pd.to_datetime(df["timestamp"])
    cat_df = pd.read_csv(cat_csv)
    cat_df["time"] = pd.to_datetime(cat_df["time"])

    # print(df)
    # print(cat_df)
    get_gamma_catalog(df,cat_df)


from obspy.clients.fdsn import Client
from obspy.core.utcdatetime import UTCDateTime
from obspy.realtime.signal import scale
# from obspy.core.stream import Stream

import matplotlib.pyplot as plt
import matplotlib.cm     as cm
import datetime as dt
import pandas as pd
import numpy as np
import json
from mpl_toolkits.axes_grid1 import make_axes_locatable
import matplotlib.gridspec as gs
import matplotlib.dates as mdates
from . import utils as ut
# import utils as ut
from matplotlib.dates import DateFormatter
from matplotlib.transforms import blended_transform_factory

class Tracer():
    def __init__(self,provider,picks):
        self.provider = provider
        self.picks = picks
    
    def plot(self,show=True):
        waveform_restrictions = self.provider.waveform_restrictions
        processing = self.provider.processing
        st = self.provider.client.get_waveforms(waveform_restrictions.network,
                                waveform_restrictions.station,
                                waveform_restrictions.location,
                                waveform_restrictions.channel,
                                waveform_restrictions.starttime,
                                waveform_restrictions.endtime)
        fig = ut.plot_multiple_picker(st,self.picks,processing )
        if show:
            plt.show()
        return fig

class Streamer():
    def __init__(self,providers,picks):
        self.providers = providers
        self.picks = picks

    def plot(self,picker,starttime,endtime,
                order,
                fontsize=6,
                show=True):

        picker_csv = self.picks[picker]
        streams = ut.get_ordered_streams(self.providers,order,
                                    starttime,endtime)
        fig = ut.get_streamer_plot(streams,picker_csv,
                                    starttime,endtime,
                                    fontsize,show)
        return fig

    def plot_by_station(self,picker,netsta,n_picks=50,
                        align="S",
                        phase_second = 2,
                        window = 15,
                        show=True):

        picker_csv = self.picks[picker]
        df = ut.get_picks(picker_csv,
                    select_networks=[netsta[0]],
                    select_stations=[netsta[1]])
        fig = ut.get_plot_by_station(self.providers,netsta,
                            df,n_picks,align,phase_second,
                            window,
                            show)
        return fig
        
if __name__ == "__main__":
    client = Client(base_url='http://sismo.sgc.gov.co:8080/')

    eqt_csv = "/home/emmanuel/EDCT/test/picks/eqt/seismonitor_picks.csv"
    sgc_csv = "/home/emmanuel/EDCT/test/picks/eqt/seismonitor_picks.csv"
    phasenet_csv = "/home/emmanuel/EDCT/test/picks/eqt/seismonitor_picks.csv"
    picks = {"EQT":eqt_csv,"PNET":phasenet_csv,"SGC":sgc_csv}

    # Trace(client,picks)
    # # client = Client(base_url='http://10.100.100.232:8091')
    # # starttime = UTCDateTime("20191224T185959")
    # starttime = UTCDateTime("20191224T190600")
    # endtime = UTCDateTime("20191224T191359")
    # st = client.get_waveforms(network="CM",station="URMC",
    #                                 location="*",
    #                                 channel="HHZ",
    #                                 starttime=starttime,
    #                                 endtime=endtime)

    # # csvs = [eqt_csv,sgc_csv,phasenet_csv]

    # # df = get_picks(phasenet_csv,starttime,endtime)
    # # print(df)
    # fig = plot_multiple_picker(st,csvs)
    # fig.savefig("/home/emmanuel/EDCT/SeisMonitor/SeisMonitor/plot/mpt.png",dpi=300)
    # # plt.show()


from concurrent.futures import process
from obspy.clients.fdsn import Client
from obspy.core.utcdatetime import UTCDateTime
from obspy.realtime.signal import scale
# from obspy.core.stream import Stream
import concurrent.futures as cf
import matplotlib.pyplot as plt
import matplotlib.cm     as cm
import datetime as dt
import pandas as pd
import numpy as np
from mpl_toolkits.axes_grid1 import make_axes_locatable
import matplotlib.gridspec as gs
import matplotlib.dates as mdates
from obspy.geodetics.base import gps2dist_azimuth
from SeisMonitor.monitor.downloader import utils as dld_ut
from matplotlib.dates import DateFormatter
from matplotlib.transforms import blended_transform_factory



####### streamer

def get_plot_by_station(providers,netsta,
                        df,n_picks=50,
                        align="P",
                        phase_second = 2,
                        window = 15,
                        show=True):

    left_padding = dt.timedelta(seconds=phase_second)
    right_padding = dt.timedelta(seconds=window)

    if align.upper() == "P":
        df = df[df["phasehint"] == "P"]
    elif align.upper() == "S":
        df = df[df["phasehint"] == "S"]

    if df.empty:
        return None
    elif len(df) > n_picks:
        df = df.sample(n_picks)
    else:
        n_picks = len(df)

    df = df.sort_values("probability",ascending=False)

    fig, ax = plt.subplots(n_picks, sharex=True,
                    gridspec_kw = {'wspace':0, 'hspace':0})
        
    trans_above = blended_transform_factory(ax[0].transData,
                                            ax[0].transAxes)
    arrowprops = dict(
            arrowstyle="->",
                facecolor="black")
    bbox = dict(boxstyle="round", fc="w")
    ax[0].annotate(align,(phase_second,1),
                    xytext=(phase_second,3),
                    xycoords=trans_above, 
                    textcoords=trans_above,
                    ha="center", va="bottom",
                    arrowprops=arrowprops, bbox=bbox)

    ax[0].annotate("Prob",(window+0.5,1),
                    xytext=(window+0.5,3),
                    xycoords=trans_above, 
                    textcoords=trans_above,
                    ha="center", va="bottom",
                    arrowprops=arrowprops, bbox=bbox)

    for i, (_, row) in enumerate(df.iterrows()):
        starttime = UTCDateTime( row["arrival_time"]) - left_padding
        endtime = starttime  + right_padding

        for provider in providers:
            client = provider.client
            try:
                st = client.get_waveforms(network=netsta[0],station=netsta[1],
                            location="*",
                            channel=row["instrument_type"]+"Z",
                            starttime=starttime,
                            endtime=endtime)
            except:
                continue

        tr = get_proc_tr(st)
        sec = tr.stats.endtime - tr.stats.starttime
        sr = tr.stats.sampling_rate
        sec_x = np.arange(0,sec+1/sr,1/sr)
        tr_y = tr.normalize().data
        ax[i].plot(sec_x,tr_y, color='black')
        ax[i].set_yticks([])

        left, width = .05, .95
        bottom, height = .05, .85
        right = left + width
        top = bottom + height
        ax[i].text(right, top, row["probability"],
            horizontalalignment='right',
            verticalalignment='top',
            transform=ax[i].transAxes)
            
    suptitle = ".".join(netsta)
    plt.xlabel('Seconds')
    # fig.xlabel('Seconds', fontsize=12)
    fig.suptitle(f'{suptitle}',fontsize=12)
    fig.tight_layout() 

    if show:
        plt.show()
    return fig

def get_streamer_plot(streams,picker_csv,
                    starttime,endtime,
                    y_fontsize=6,show=True):
    n_traces = len(list(streams.keys()))
    fig, ax = plt.subplots(n_traces, sharex=True,
                gridspec_kw = {'wspace':0, 'hspace':0})
    
    if n_traces == 1:
        ax = [ax]


    
    for i,(strid,st) in enumerate(streams.items()):
        tr = get_proc_tr(st)
        STARTTIME = mdates.date2num(starttime.datetime)

        x = tr.times("matplotlib")-STARTTIME
        ax[i].plot(x, tr.data, "k-")
        ax[i].xaxis_date()
        ax[i].set_xlim([min(x),max(x)])
        ymin, ymax = ax[i].get_ylim()
        ax[i].set_yticks([])
        ax[i].set_ylabel(strid,rotation=0,labelpad=24,fontsize=y_fontsize)

        df = get_picks(picker_csv,starttime,endtime,
                    select_stations=[tr.stats.station])
        for j, (_, row) in enumerate(df[::-1].iterrows()):
            pick = mdates.date2num(row['arrival_time'])-STARTTIME
            if row['phasehint'].upper() == 'P':
                color = "blue" 
            elif row['phasehint'].upper() == 'S':
                color = "red"
            
            ax[i].vlines(pick, ymin, ymax, color=color, 
                        linewidth=5, 
                        label=row['phasehint'])
            # ax[i].set_facecolor('lightgray')

    text = "starttime: " + starttime.strftime("%Y-%m-%d %H:%M:%S.%f")+\
            "\nendtime : " + endtime.strftime("%Y-%m-%d %H:%M:%S.%f")
    props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
    # ax[0].text(0.8, 5, text,
    ax[0].text(0.45, 1.2, text,
                horizontalalignment='left',
                verticalalignment='top',
                transform=ax[0].transAxes,
                backgroundcolor= "white",
                fontdict={"fontsize":9},
                bbox=props
                )

    ax[i].set_xlabel(f"dt", size=16)
    # fig.autofmt_xdate()
    formatter = DateFormatter('%H:%M:%S')
    plt.gcf().axes[i].xaxis.set_major_formatter(formatter)  

    plt.tight_layout()

    if show:
        plt.show()

    return fig

def get_ordered_streams(providers,order,
                        starttime,endtime,
                        n_processor=None):
    ordered_info,providers = get_ordered_info(providers,order)

    def get_client_waveforms_by_thread(netsta):
        strid = ".".join(netsta)
        bulk = (netsta[0],netsta[1],waveform_restrictions.location,
                waveform_restrictions.channel,starttime,endtime)
        st,_,_ = dld_ut.get_client_waveforms(client,bulk,
                                waveform_restrictions,
                                processing  )
        if len(st)==0:
            return 

        return (strid,st)

    for provider in providers:
        provider.waveform_restrictions.starttime = starttime
        provider.waveform_restrictions.endtime = endtime
        client = provider.client
        waveform_restrictions = provider.waveform_restrictions
        bulk_info = waveform_restrictions.bulk_info
        processing = provider.processing

        if n_processor == 1:
            result = []
            for netsta in waveform_restrictions.bulk_info:
                st = get_client_waveforms_by_thread(netsta)
                result.append(st)
        else:
            with cf.ThreadPoolExecutor(n_processor) as executor:
                result = executor.map(get_client_waveforms_by_thread,bulk_info)
                result = list(result)
        
        result = list(filter(None, result))
        result = dict(list(result))

    st_keys = list(result.keys())        

    streams = {}
    for netsta,row in ordered_info.iterrows():
        strid = ".".join(netsta)

        if strid in st_keys:
            st = result[strid]
        else:
            print("no",strid)
            continue

        streams[strid] = st

    # print(streams)

    return streams

def get_ordered_info(providers,order=None):
    inv,json_info,providers,sod = dld_ut.get_merged_inv_and_json(providers)
    
    json_info = pd.DataFrame.from_dict(json_info, orient='index')
    json_info["station"] = json_info.index
    json_info = json_info.set_index(["network","station"])

    if order == None:
        return json_info,providers
    elif isinstance(order[0],str):
        lat,lon,_ = json_info.loc[order,"coords"]
    elif isinstance(order[0],float):
        lat,lon = order

    # json_info = json_info.reset_index(drop=True)
    distance_func = lambda y: gps2dist_azimuth(y[0],y[1],lat,lon)[0]
    json_info["r"] = json_info["coords"].apply(distance_func)
    
    json_info = json_info.sort_values(by="r")
    return json_info,providers

### multiple picker trace
def get_picks(csv,starttime=None,endtime=None,
                select_networks=[],select_stations=[],
                filter_networks=[],filter_stations=[]):
    "filter dataframe data"

    df = pd.read_csv(csv,parse_dates=['arrival_time'])

    if isinstance(starttime, UTCDateTime):
        starttime = starttime.datetime
    if isinstance(endtime, UTCDateTime):
        endtime = endtime.datetime

    if isinstance(df["arrival_time"].dtype, pd.core.dtypes.dtypes.DatetimeTZDtype):
        raise Exception("revise el dataframe. quitele el +00:00")
        # sed -i 's/+00:00,/,/g' /mnt/almacenamiento/ecastillo/data/multiple_picker_trace/phasenet.csv
    else:
        pass

    if starttime != None:
        df = df[df["arrival_time"] >= starttime ]
    if endtime != None:
        df = df[df["arrival_time"] <= endtime ]

    if select_networks:
        df = df[ df["network"].isin(select_networks) ]
    if select_stations:
        df = df[ df["station"].isin(select_stations) ]
    if filter_networks:
        df = df[ ~df["network"].isin(filter_networks) ]
    if filter_stations:
        df = df[ ~df["station"].isin(filter_stations) ]
    
    return df

def get_proc_tr(st,
        processing=None):
    "returns the first processed trace"
    
    if processing == None:
        processed = False
        comment = ""
    else:
        st,processed, comment = processing.run(st) 

    st.trim(min([tr.stats.starttime for tr in st]),
             max([tr.stats.endtime for tr in st]), 
             pad=True, fill_value=0)
    tr = st[0]
    return tr

def get_multiple_picker_figure(n_pickers,tr_h = 0.7):
    fig = plt.figure(figsize=(12,12))
    gs1 = gs.GridSpec(1, 1, figure=fig)
    gs1.update(left=0.12, right=0.9, wspace=0.05)

    picker_h = 1 - tr_h
    picker_r = [round(x/n_pickers,3) for x in [picker_h]*n_pickers]
    height_ratios = [tr_h] + picker_r

    gss = gs.GridSpecFromSubplotSpec(len(height_ratios), 1,subplot_spec=gs1[0], hspace=0.0,
                                    height_ratios=height_ratios)

    ax0 = fig.add_subplot(gss[0])
    ax0.tick_params(axis="x", labelbottom=0)
    ax0.set_yticks([])
    time_axes = [ax0]
    for i in range(1,len(height_ratios)):
        ax = fig.add_subplot(gss[i],sharex=ax0)

        if i != len(height_ratios) -1:
            ax.tick_params(axis="x", labelbottom=0)

        ax.set_yticks([])
        time_axes.append(ax)

    fig.autofmt_xdate()


    gs2 = gs.GridSpec(1, 1)
    gs2.update(left=0.92, right=0.96, hspace=0.2)
    gss2 = gs.GridSpecFromSubplotSpec(1, 2,subplot_spec=gs2[0], hspace=0.5,width_ratios=[1,1])
    ax4 = fig.add_subplot(gss2[0])
    ax4.set_yticks([])
    ax4.set_xticks([])
    ax5 = fig.add_subplot(gss2[1],sharey=ax4)
    ax5.set_yticks([])
    ax5.set_xticks([])
    ax5.text(1.1,0.5,"Probability", size=10,
            verticalalignment='center', rotation=270)

    prob_axes = [ax4,ax5]
    return fig, time_axes, prob_axes

def plot_multiple_picker(st,info,
                        processing = None,
                        tr_h =0.7
                        ):

    len_pickers = len(list(info.keys()))
    fig,ax,pax = get_multiple_picker_figure(len_pickers,tr_h)

    len_axes = len(ax)

    tr = get_proc_tr(st,processing )

    network = tr.stats.network
    station = tr.stats.station
    location = tr.stats.location
    channel = tr.stats.channel
    starttime = tr.stats.starttime
    endtime = tr.stats.endtime
    STARTTIME = mdates.date2num(starttime.datetime)

    x = tr.times("matplotlib")-STARTTIME
    ax[0].plot(x, tr.data, "k-")
    text = ".".join((network,station,
                 
                    location,channel))
    text = text.rjust(len(text) + 18)
    text += "\nstarttime: " + starttime.strftime("%Y-%m-%d %H:%M:%S.%f")+\
            "\nendtime : " + endtime.strftime("%Y-%m-%d %H:%M:%S.%f")

    props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
    # ax[0].text(.03, .95, text,
    ax[0].text(0.72, 0.13, text,
                horizontalalignment='left',
                verticalalignment='top',
                transform=ax[0].transAxes,
                backgroundcolor= "white",
                fontdict={"fontsize":9},
                bbox=props)
    ax[0].xaxis_date()
    ax[0].set_xlim([min(x),max(x)])
    ymin, ymax = ax[0].get_ylim()

    ax[len_axes-1].set_xlabel(f"dt", size=16)

    # https://stackoverflow.com/questions/8342549/matplotlib-add-colorbar-to-a-sequence-of-line-plots
    # Pcmap = cm.winter.reversed()
    # Scmap = cm.autumn.reversed()
    # Pcmap = cm.get_cmap("winter",10)
    Pcmap = cm.get_cmap("Blues",10)
    # Pcmap = Pcmap.reversed()
    Scmap = cm.get_cmap("Reds",10)
    # Scmap = Scmap.reversed()
    for i,(picker,csv) in enumerate(info.items(),1):
        df = get_picks(csv,starttime,endtime,
                        select_stations=[tr.stats.station])
        for j, (_, row) in enumerate(df.iterrows()):
            pick = mdates.date2num(row['arrival_time'])-STARTTIME
            if row['phasehint'].upper() == 'P':
                color = Pcmap(row['probability']-0.001) # -0.001 removes a bug in Cbar 
            elif row['phasehint'].upper() == 'S':
                color = Scmap(row['probability']-0.001)

            ax[i].vlines(pick, ymin, ymax, color=color, 
                            # linewidth=0.7, 
                            linewidth=1, 
                            label=row['phasehint'])
            ax[i].set_facecolor('lightgray')
            ax[i].set_yticks([])
            ax[i].set_ylabel(picker,rotation=0,labelpad=24,fontsize=16)

    fig.autofmt_xdate()

    Pclb = fig.colorbar(cm.ScalarMappable(cmap=Pcmap), cax=pax[0],orientation="vertical")
    Pclb.ax.tick_params(labelsize=1)
    Pclb.ax.set_title('P',fontsize=18)
    Sclb = fig.colorbar(cm.ScalarMappable(cmap=Scmap), cax=pax[1],orientation="vertical")
    Sclb.ax.set_title('S',fontsize=18)
    # Sclb.ax.set_title('Probability',fontsize=10)

    fig.autofmt_xdate()
    return fig

if __name__ == "__main__":
    # client = Client(base_url='http://10.100.100.232:8091')
    client = Client(base_url='http://sismo.sgc.gov.co:8080/')
    # starttime = UTCDateTime("20191224T185959")
    starttime = UTCDateTime("20191224T190600")
    endtime = UTCDateTime("20191224T191359")
    st = client.get_waveforms(network="CM",station="URMC",
                                    location="*",
                                    channel="HHZ",
                                    starttime=starttime,
                                    endtime=endtime)

    eqt_csv = "/home/emmanuel/EDCT/test/picks/eqt/seismonitor_picks.csv"
    sgc_csv = "/home/emmanuel/EDCT/test/picks/eqt/seismonitor_picks.csv"
    phasenet_csv = "/home/emmanuel/EDCT/test/picks/eqt/seismonitor_picks.csv"
    # csvs = [eqt_csv,sgc_csv,phasenet_csv]
    csvs = {"EQT":eqt_csv,"PNET":phasenet_csv,"SGC":sgc_csv}

    # df = get_picks(phasenet_csv,starttime,endtime)
    # print(df)
    fig = plot_multiple_picker(st,csvs)
    fig.savefig("/home/emmanuel/EDCT/SeisMonitor/SeisMonitor/plot/mpt.png",dpi=300)


import os
import glob
from posixpath import basename
import shutil
import numpy as np
from typing import Union
import concurrent.futures as cf
from obspy.core.event.base import CreationInfo
from obspy.core.event.catalog import Catalog, read_events
from obspy.geodetics.base import gps2dist_azimuth
from obspy.io.nlloc.core import read_nlloc_hyp
from obspy.core.event.base import CreationInfo
from obspy.core.event.resourceid import ResourceIdentifier
from obspy import UTCDateTime
import subprocess
from tqdm import tqdm
from . import utils as ut
from SeisMonitor import utils as sut
from SeisMonitor.core import utils as scut
from SeisMonitor.monitor.locator import utils as slut

CORE_NLLOC = os.path.join(os.path.dirname(__file__),"core")
NLLOC_path = os.path.join(CORE_NLLOC,"NonLinLoc-main")
src_path = os.path.join(NLLOC_path,"src")
bin_path = os.path.join(src_path,"bin")
vel2grid_exe_path = os.path.join(bin_path,"Vel2Grid")
grid2time_exe_path = os.path.join(bin_path,"Grid2Time")
nll_exe_path = os.path.join(bin_path,"NLLoc")


#https://github.com/SEISAN-EARTHQUAKE-ANALYSIS-SOFTWARE/SEISAN-manual/blob/master/appendix/nordic-format.tex seisan format
class NLLoc():
    def __init__(self,
                agency:str,
                region:list,
                vel_model:slut.VelModel,
                stations:slut.Stations,
                delta_in_km: float=2,
                kwargs_for_trans:dict={},
                kwargs_for_vel2grid:dict={},
                kwargs_for_grid2time:dict={},
                kwargs_for_time2loc:dict={},
                tmp_folder:str = os.getcwd(),
                exhaustively:bool = False,
                search_in_degrees:list = [],
                rm_attempts:bool = False
                ):

        self.agency = agency
        self.region = region
        self.vel_model = vel_model
        self.stations = stations
        self.basic_inputs = slut.LocatorBasicInputs(vel_model=vel_model,
                                                    stations=stations)
        self.delta_in_km = delta_in_km
        self.kwargs_for_trans = kwargs_for_trans
        self.kwargs_for_vel2grid = kwargs_for_vel2grid
        self.kwargs_for_grid2time = kwargs_for_grid2time
        self.kwargs_for_time2loc = kwargs_for_time2loc
        self.tmp_folder = tmp_folder
        self.rm_attempts = rm_attempts

        self.exhaustively = exhaustively
        if exhaustively:
            #     lon_distance_in_deg = abs(region[0]-region[1])
            #     lat_distance_in_deg = abs(region[2]-region[3])
            #     r = np.sqrt(lon_distance_in_deg**2 + lat_distance_in_deg**2 )
            #     self.search_in_degrees = np.linspace(1/111,r/2,5)[::-1]
            if not search_in_degrees:
                self.search_in_degrees = list(reversed(np.arange(1,4,0.5)))
            else:
                self.search_in_degrees = search_in_degrees

    def __initialize(self,write_nlloc_files=True):
        ### inputs
        self.vel_model_path = os.path.join(self.tmp_folder,"time_grid","vel_model.dat")
        self.station_path = os.path.join(self.tmp_folder,"time_grid","station.dat")
        sut.isfile(self.vel_model_path,overwrite=True)
        self.basic_inputs.vel_model.to_nlloc(self.vel_model_path)
        sut.isfile(self.station_path,overwrite=True)
        self.basic_inputs.stations.to_nlloc(self.station_path)

        self.grid_folder_out = os.path.join(self.tmp_folder,"time_grid","model","layer")
        self.time_folder_out = os.path.join(self.tmp_folder,"time_grid","time","layer")
        self.loc_folder_out = os.path.join(self.tmp_folder,"time_grid","loc","SeisMonitor")
        self.p_control_file_out = os.path.join(self.tmp_folder,"time_grid","p_nlloc.in")
        self.s_control_file_out = os.path.join(self.tmp_folder,"time_grid","s_nlloc.in")

        grid_args = self._prepare_grid_args()

        if "trans" in list(self.kwargs_for_trans.keys()):
            grid_args["trans"] = self.kwargs_for_trans["trans"]
        if "grid" in list(self.kwargs_for_vel2grid.keys()):
            grid_args["velgrid"] = self.kwargs_for_vel2grid["grid"]
        if "grid" in list(self.kwargs_for_grid2time.keys()):
            grid_args["locgrid"] = self.kwargs_for_grid2time["grid"]

        gen_control = ut.GenericControlStatement(trans=grid_args["trans"])
        gen_vel2grid = ut.Vel2Grid(vel_path=self.vel_model_path,
                    grid_folder_out=self.grid_folder_out,
                    grid=grid_args["velgrid"])
        p_grid2time = ut.Grid2Time(station_path=self.station_path,
                            grid_folder_out=self.grid_folder_out,
                            time_folder_out=self.time_folder_out,
                            phase="P")
        s_grid2time = ut.Grid2Time(station_path=self.station_path,
                            grid_folder_out=self.grid_folder_out,
                            time_folder_out=self.time_folder_out,
                            phase="S")

        self.catalog_path = os.path.join(self.tmp_folder,"time_grid","catalog.out")
        sut.isfile(self.catalog_path,overwrite=True)
        open(self.catalog_path, "w")
        gen_time2loc = ut.Time2Loc(catalog=[self.catalog_path,"SEISAN"],
                    grid = grid_args["locgrid"],
                    time_folder_out=self.time_folder_out,
                    loc_folder_out=self.loc_folder_out)

        def write_each_nlloc_file(nlloc_and_out):
            nlloc,out = nlloc_and_out
            nlloc.write(out,
                    vel2grid=True,
                    grid2time=True,
                    time2loc=True)
            return True


        p_nlloc = ut.NLLocControlFile(gen_control,gen_vel2grid,
                                    p_grid2time,gen_time2loc)
        s_nlloc = ut.NLLocControlFile(gen_control,gen_vel2grid,
                        s_grid2time,gen_time2loc)
        nlloc_control_files = [(p_nlloc,self.p_control_file_out),
                                (s_nlloc,self.s_control_file_out)]

        if write_nlloc_files:
            ## The parallelization doesn't work properly, therefore I only use max_workers=1. I am too lazy to change the code. 
            with cf.ThreadPoolExecutor(max_workers=1) as executor:
                executor.map(write_each_nlloc_file,nlloc_control_files)

        self.nll_control_file = p_nlloc

    def _prepare_grid_args(self):
        lonw,lone,lats,latn,zmin,zmax = self.region

        c_lat = (lats+latn)/2
        c_lon = (lonw+lone)/2

        x,_,_ = gps2dist_azimuth(lats,lonw,lats,c_lon)
        x = x/1e3
        x_num = int(x*2/self.delta_in_km)
        y,_,_ = gps2dist_azimuth(lats,lonw,c_lat,lonw)
        y = y/1e3
        y_num = int(y*2/self.delta_in_km)

        z_num = int((zmax-zmin)/self.delta_in_km)

        args = {
                "trans":["SIMPLE",c_lat,c_lon,0],
                # "trans":["LAMBERT","WGS-84",lats,lonw,0,15,0],
                "velgrid":[x_num,y_num,z_num,
                            -round(x,2),-round(y,2),round(zmin,2),
                            self.delta_in_km,self.delta_in_km,
                            self.delta_in_km,"SLOW_LEN"],
                "locgrid":[x_num,y_num,z_num,
                            -round(x,2),-round(y,2),round(zmin,2),
                            self.delta_in_km,self.delta_in_km,
                            self.delta_in_km,
                            "PROB_DENSITY","SAVE"]
                            }
        return args

    def download(self):
        if not os.path.isdir(ut.NLLOC_path):
            ut.download_nlloc()
        else:
            print(f"NonLinLoc is located in {ut.NLLOC_path}")

    def compute_travel_times(self):
        self.__initialize()

        sut.printlog("info","NLLoc:Vel2Grid", "Running")
        # vel2grid = subprocess.call(f"{vel2grid_exe_path} {self.p_control_file_out} > /dev/null")
        vel2grid = subprocess.call(f"{vel2grid_exe_path} {self.p_control_file_out}",shell=True)
        sut.printlog("info","NLLoc:Grid2Time:P", "Running")
        grid2time = subprocess.call(f"{grid2time_exe_path} {self.p_control_file_out}",shell=True)
        sut.printlog("info","NLLoc:Grid2Time:S", "Running")
        grid2time = subprocess.call(f"{grid2time_exe_path} {self.s_control_file_out}",shell=True)

    def _locate(self,
                catalog:Union[Catalog,str],
                nlloc_out_folder:str,
                out_filename:str = "locations.xml",
                out_format:str = "SC3ML"):

        if isinstance(catalog,Catalog):
            pass
        else:
            catalog = read_events(catalog)

        nlloc_inp = os.path.join(nlloc_out_folder,"catalog_input.inp")
        nlloc_folder = os.path.join(nlloc_out_folder,"nlloc","SeisMonitor")
        nlloc_out = os.path.join(nlloc_out_folder,out_filename)
        nlloc_control = os.path.join(nlloc_out_folder,"loc.in")

        sut.isfile(nlloc_inp,overwrite=True)
        catalog.write(nlloc_inp,format="NORDIC")

        picks_from_unproc_catalog = slut.get_picks(catalog)
        # arrivals_from_unproc_catalog = slut.get_picks(catalog)

        try:
            nlloc_control_file = self.nll_control_file
        except:
            self.__initialize(False)
            nlloc_control_file = self.nll_control_file

        nlloc_control_file.time2loc.catalog = " ".join((nlloc_inp,"SEISAN")) 
        nlloc_control_file.time2loc.loc_folder_out = nlloc_folder 
        nlloc_control_file.write(nlloc_control)

        sut.printlog("info","NLLoc:NLLoc", "Running")
        subprocess.call(f"{nll_exe_path} {nlloc_control}",shell=True)
        
        _nll_out = nlloc_folder+"*.hyp"
        all_events = []
        for path in tqdm(glob.glob(_nll_out)):
            basename = os.path.basename(path)
            date = basename.split(".")[1]
            if (date == "sum") or (date=="last.hyp"):
                continue
            else:
                try:
                    catalog = read_nlloc_hyp(path,format="NORDIC")
                except:
                    print(f"Unread: {path}")
                    continue

                for ev in catalog.events:
                    ori_pref  = ev.preferred_origin()
                    ori_pref = scut.add_aditional_origin_info(ori_pref,
                                    agency=self.agency,
                                    method_id="NLLOC",
                                    earth_model_id=self.vel_model.model_name)
                    ev.preferred_origin_id = ori_pref.resource_id.id

                    ev = slut.changing_picks_info(ev,picks_from_unproc_catalog)
                    ev = scut.add_aditional_event_info(ev,agency=self.agency)
                    all_events.append(ev)

        catalog = Catalog(events = all_events)
        catalog = scut.add_aditional_catalog_info(catalog,agency=self.agency)

        sut.isfile(nlloc_out)
        catalog.write(nlloc_out,format=out_format)
        sut.printlog("info","NLLoc:NLLoc", f"Finished. See your results in {nlloc_out}")
        return catalog

    def _iterlocate(self,
                catalog:Union[Catalog,str],
                nlloc_out_folder:str,
                out_filename:str = "locations.xml",
                out_format:str = "SC3ML",
                degrees=[4,3,2.5,2,1.5,1],
                rm_attempts=False
                ):     
        nlloc_out = os.path.join(nlloc_out_folder,out_filename)
        reloc_catalog = self._locate(catalog,nlloc_out_folder,
                    out_filename="base.xml",out_format="SC3ML")
        
        good_evs = []
        bad_catalog = reloc_catalog
        iter = 0
        while True:
            good_events,bad_events = slut.get_bad_and_good_events(bad_catalog)
            good_evs.append(good_events)
            degree = degrees[iter]
            if not bad_events:
                if iter == 0:
                    print(f"convergence without iterate")
                else:
                    print(f"convergence in {iter} iterations, only stations with distance < {degrees[iter-1]} degrees")
                break
            else:
                bad_events = slut.filter_arrivals_by_distance(bad_events,degree)
                
                if not bad_events:
                    break
                
                bad_catalog = Catalog(bad_events)
                tmp_path = os.path.join(nlloc_out_folder,"tmp",f"deg_{degree}")
                bad_catalog = self._locate(bad_catalog,tmp_path,
                    out_filename=f"{iter}.xml",out_format="SC3ML")
                iter +=1
        good_evs = [ y for x in good_evs for y in x]

        if not good_evs:
            pass
        else:
            reloc_catalog.events = good_evs

        sut.isfile(nlloc_out)
        reloc_catalog.write(nlloc_out,
                    format=out_format)
        sut.printlog("info","NLLoc:NLLoc", f"Finished. See your results in {nlloc_out}")
        
        if rm_attempts:
            shutil.rmtree(os.path.join(nlloc_out_folder,"tmp"))

        return catalog

    def locate(self,
                catalog:Union[Catalog,str],
                nlloc_out_folder:str,
                out_filename:str = "locations.xml",
                out_format:str = "SC3ML"
                ):
        if self.exhaustively:
            self._iterlocate(catalog=catalog,
                            nlloc_out_folder=nlloc_out_folder,
                            out_filename=out_filename,
                            out_format=out_format,
                            degrees=self.search_in_degrees,
                            rm_attempts=self.rm_attempts)
        else:
            self._locate(catalog=catalog,
                            nlloc_out_folder=nlloc_out_folder,
                            out_filename=out_filename,
                            out_format=out_format)


from locale import D_FMT
import os
import datetime as dt
import pandas as pd
import obspy
from obspy.core.inventory.inventory import Inventory
from obspy.io.xseed.parser import Parser
from SeisMonitor.utils import printlog,isfile
from obspy.core.utcdatetime import UTCDateTime
import warnings
warnings.filterwarnings('ignore')
### EQTransformer util associator

SeisMonitor_columns = ["pick_id","arrival_time","probability","phasehint",
                    "network","station","location","instrument_type","author",
                    "creation_time","event_start_time","event_end_time",
                    "detection_probability","snr","station_lat","station_lon",
                    "station_elv","file_name"]

EQT_COLUMNS = ["file_name","network",'station',
    'instrument_type','station_lat','station_lon','station_elv',
    'event_start_time','event_end_time','detection_probability',
    'detection_uncertainty','p_arrival_time','p_probability',
    'p_uncertainty','p_snr','s_arrival_time',
    's_probability','s_uncertainty','s_snr']

GaMMA_picks_columns = ["id","timestamp","type","prob","amp"]

paz_wa = {'sensitivity': 2800, 'zeros': [0j], 'gain': 1,
          'poles': [-6.2832 - 4.7124j, -6.2832 + 4.7124j]}

def get_picks_GaMMa_df(picks,response,compute_amplitudes=True
                        ,p_window=10,
                         s_window=5,waterlevel=10):
    df = pd.read_csv(picks)
    date_cols = ["arrival_time","creation_time",
                "event_start_time","event_end_time"]
    df[date_cols] = df[date_cols].apply(pd.to_datetime)

    if compute_amplitudes:
        df = get_seismonitor_amplitudes(df,response,
                                        p_window=p_window,
                                        s_window=s_window,
                                        waterlevel = waterlevel)
    else:
        df = df

    id_func = lambda x: x.split("-")[-1]
    df["id"] = df["pick_id"].apply(id_func)
    df = df.rename(columns={"arrival_time":"timestamp",
                        "probability":"prob",
                        "phasehint":"type",
                        "amplitude":"amp"})
    df = df.drop_duplicates(ignore_index=True)
    return df

def get_stations_GaMMA_df(response):
    id_list = []
    station_list = []
    longitude_list = []
    latitude_list = []
    elevation_list = []
    for network in response:
        for station in network:
            net = network.code
            sta = station.code
            elv = station.elevation
            lat = station.latitude
            lon = station.longitude
            # components = []
            # sensitivities = []
            # for channel in station:
            #     component = channel.code[-1]
            #     sensitivity = channel.response.get_paz().normalization_factor
                
            #     components.append(component)
            #     sensitivities.append(sensitivity)


            # unit = channel.response.response_stages[0].input_units
            inst = station[0].code[:-1]
            loc = station[0].location_code

            scr_id = ".".join((net,sta,loc,inst))

            id_list.append(scr_id)
            station_list.append(sta)
            longitude_list.append(lon)
            latitude_list.append(lat)
            elevation_list.append(elv)

    df = {"id":id_list,
        "longitude":longitude_list,
        "latitude":latitude_list,
        "elevation(m)":elevation_list,
        "station_name":station_list,
        }
    df = pd.DataFrame(df)
    df = df.drop_duplicates(subset="id",ignore_index=True)
    # print(df)
    return df

def get_paz_from_response(seed_id,response,
                            datetime=None):
    if isinstance(response,Parser):
        try:
            paz = response.get_paz(seed_id,datetime)
        except:
            return None

    elif isinstance(response,Inventory):
        try:
            response = response.get_response(seed_id,
                                    datetime)
        
            paz_stage = response.get_paz()
        except:
            return None

        sensitivity = response.instrument_sensitivity.value
        gain = paz_stage.normalization_factor
        poles = paz_stage.poles
        zeros = paz_stage.zeros

        paz = {'poles': poles,
                'zeros': zeros,
                'gain': gain,
                'sensitivity': sensitivity}
    return paz

def get_amplitudes_from_pick(st,picktime,phasehint,
                            p_window=10,s_window=5):

    _st = st.copy()

    if phasehint.upper() == "P":
        trimmedtime = dt.timedelta(seconds=p_window)
        _st.trim(UTCDateTime(picktime),UTCDateTime(picktime)+trimmedtime)

        if len(_st)== 0:
            tr_z = _st.select(component="Z")[0]
            ampl = max(abs(tr_z.data))
        else:
            tr = st[0]
            ampl = max(abs(tr.data))

    elif phasehint.upper() == "S":
        trimmedtime = dt.timedelta(seconds=s_window)
        _st.trim(UTCDateTime(picktime),UTCDateTime(picktime)+trimmedtime)

        if len(_st)== 0:
            tr_n = _st.select(component="N")[0]
            ampl_n = max(abs(tr_n.data))
            tr_e = _st.select(component="E")[0]
            ampl_e = max(abs(tr_e.data))
            ampl = max(ampl_n, ampl_e)
        else:
            tr = st[0]
            ampl = max(abs(tr.data))

    return ampl

def get_amplitudes_from_local_st(file_name,df,response,
                                p_window=10,s_window=5,
                                waterlevel=10):
    st = obspy.read(file_name)

    for tr in st:
        paz = get_paz_from_response(tr.id,response,
                                tr.stats.starttime)
        if paz == None:
            print(f"\t->No response found: {tr.id}-{tr.stats.starttime}")

        tr.simulate(paz_remove=paz, 
                    paz_simulate=paz_wa, 
                    water_level=waterlevel)


    ampl_func = lambda x: get_amplitudes_from_pick(st,x.arrival_time,
                                            x.phasehint,p_window,s_window)
    df["amplitude"] = df.apply(ampl_func,axis=1)

    return df
    
def get_seismonitor_amplitudes(df,response,out=None,
                         p_window=10,s_window=5,
                        waterlevel=10):
    # df = pd.read_csv(picks)
    date_cols = ["arrival_time","creation_time",
                "event_start_time","event_end_time"]
    df[date_cols] = df[date_cols].apply(pd.to_datetime)

    gdf = df.groupby(by="file_name")
    dfs = []
    for file_name,df in gdf.__iter__():
        df = get_amplitudes_from_local_st(file_name,df,response,
                        p_window,s_window,waterlevel)
        dfs.append(df)

    df = pd.concat(dfs)

    fourth_column = df.pop('amplitude')
    df.insert(3, 'amplitude', fourth_column)

    df = df.sort_values(by="arrival_time",ignore_index=True)
    
    if out != None:
        isfile(out)
        df.to_csv(out,index=False)

    return df

def link(p_row,s_df,tt,st=2,et=4):
    p_time = p_row.arrival_time
    s_minus_p = (s_df["arrival_time"]-p_time).dt.total_seconds()
    s_df["tt"] = s_df["arrival_time"][abs(s_minus_p) <= tt]
    s_df = s_df.dropna(subset=['tt'])

    s_df = s_df.sort_values(by=["tt"],ignore_index=True)
    # print(s_df["tt"])
    if len(s_df) >= 1:
        p_row["s_arrival_time"] = s_df["tt"][0]
        p_row["s_probability"] = s_df["probability"][0]
        p_row["s_pick_id"] = s_df["pick_id"][0]

        p_row['event_start_time'] = p_time-dt.timedelta(seconds=st)
        p_row['event_end_time'] = p_time+dt.timedelta(seconds=et)
        p_row['detection_probability'] = (p_row["probability"] + p_row["s_probability"])/2
        
    else:
        p_row["s_arrival_time"] = None
        p_row["s_probability"] = None
        p_row["s_pick_id"] = None
    
        p_row['event_start_time'] = p_time-dt.timedelta(seconds=st)
        p_row['event_end_time'] = p_time+dt.timedelta(seconds=et*1)
        p_row['detection_probability'] = p_row["probability"]/2
    p_row['file_name'] = p_row.mseed_name
    p_row['station_lat'] = p_row.station_lat
    p_row['station_lon'] = p_row.station_lon
    p_row['station_elv'] = p_row.station_elv
    return p_row

def make_eqt_dirs(eqt_df,folder):
    dfbystation = eqt_df.groupby(by=["station"])

    for name,df in dfbystation.__iter__():
        dirname = name + "_outputs"
        dirpath = os.path.join(folder,dirname)

        if not os.path.isdir(dirpath):
            os.makedirs(dirpath)
        
        filepath = os.path.join(dirpath,"X_prediction_results.csv")
        df.to_csv(filepath,index=False)

def link_seismonitor_phases(df,tt=30,st=2,et=4):
    """
    For each P pick, it searchs the corresponding S pick
    according tt parameter.
    Parameters:
    -----------
    df: pd.DataFrame
        seismonitor dataframe
    tt: float
        Each P pick search S picks in a tt-second radius.
        If there are more than one S pick, it chooses the closest.
    st: float
        Seconds before the P pick. It is used to set up
        the event start time
    et: float
        Seconds after the S pick. It is used to set up
        the event end time. If there isn't P pick, the event
        end time is configured et*2-seconds adfter the P phase time.
    Returns:
    --------
    df: pd.DataFrame
        P and S phases are linked by
    """
    columns = ["pick_id","arrival_time","probability",'phasehint','network',
             'station', 'location','instrument_type','author', 
             "station_lat","station_lon","station_elv",
            'creation_time',"mseed_name"]

    df = df [columns]
    df["arrival_time"] = pd.to_datetime(df["arrival_time"])
    dfbystation = df.groupby(by=["station"])

    eqt_df = pd.DataFrame()
    for i,df in dfbystation.__iter__():
    
        p_df = df[df["phasehint"] == "P"]
        s_df = df[df["phasehint"] == "S"]
        
        for j,row in p_df.iterrows():
            p_row = link(row,s_df,tt,st,et)
            eqt_df = eqt_df.append(p_row,ignore_index=True)

    eqt_df["p_snr"] = None
    eqt_df["p_uncertainty"] = None
    eqt_df["s_snr"] = None
    eqt_df["s_uncertainty"] = None
    eqt_df["detection_uncertainty"] = None

    eqt_df = eqt_df.rename(columns={"arrival_time":"p_arrival_time",
                            "probability":"p_probability"
                            })
    eqt_df = eqt_df[EQT_COLUMNS]
    return eqt_df

def link_eqt_phases(df):

    eqt_and_seismonitor_cols = ["file_name","network","station","instrument_type",
                "station_lat","station_lon","station_elv","event_start_time",
                "event_end_time","detection_probability"]

    p_df = df[df["phasehint"] == "P"]
    s_df = df[df["phasehint"] == "S"]

    df = p_df.merge(s_df,how="outer",on=eqt_and_seismonitor_cols,suffixes=("_p","_s"))

    df["p_uncertainty"] = None
    df["s_uncertainty"] = None
    df["detection_uncertainty"] = None

    renaming = {'pick_id_p':"p_pick_id", 'arrival_time_p':"p_arrival_time",
                 'probability_p':"p_probability", 'phasehint_p':"p_phasehint",
                 'location_p':"p_location", 'author_p':"p_author",
                'creation_time_p':"p_creation_time", 
                 'snr_p':"p_snr", 'pick_id_s':"s_pick_id", 
                 'arrival_time_s':"s_arrival_time",'probability_s':"s_probability",
                  'phasehint_s':"s_phasehint", 'location_s':"s_location",
                   'author_s':"s_location",'creation_time_s':"s_creation_time",
                'snr_s':"s_snr"}
    df = df.rename(columns=renaming)
    df = df[EQT_COLUMNS]
    return df

def seismonitor_picks_to_eqt_fmt(seismonitor_picks,eqt_folder,
                            tt=30,st=2,et=4):

    df = pd.read_csv(seismonitor_picks)
    date_cols = ["arrival_time","creation_time",
                "event_start_time","event_end_time"]
    df[date_cols] = df[date_cols].apply(pd.to_datetime)

    gdf = df.groupby(by="author")
    eqts = []
    for author,df in gdf.__iter__():
        if author == "EQTransformer":
            eqt = link_eqt_phases(df)
        else:
            eqt = link_seismonitor_phases(df,tt,st,et)
        eqts.append(eqt)
    eqt = pd.concat(eqts)

    make_eqt_dirs(eqt,eqt_folder)
    printlog('info','pnet_to_eqt_fmt',
                f'ok')



"""
 * @author Emmanuel David Castillo Taborda
 * @email ecastillot@unal.edu.co
 * @create date 2021-03-03 23:51:21
 * @modify date 2021-06-02 04:31:09
 * @desc [description]
"""

# import sys
# AIpicker_path = "/home/ecastillo/AIpicker"
# sys.path.insert(0,AIpicker_path)

import datetime as dt
import time
import os
import ast
import json
import pandas as pd
import concurrent.futures
from obspy import UTCDateTime
from obspy.core.event.base import CreationInfo
from obspy.core.event.resourceid import ResourceIdentifier
from obspy.core.event.catalog import read_events
from obspy.core.event import Catalog


def add_aditional_origin_info(origin,
				agency = None,
				region = None,
				method_id = None,
				earth_model_id = None,
				evaluation_mode = "automatic",
				evaluation_status = "preliminary"):

	if region != None:
		origin.region = region
	if earth_model_id != None:
		origin.earth_model_id = ResourceIdentifier(id=earth_model_id)
	if method_id != None:
		origin.method_id = ResourceIdentifier(id=method_id)
		
	origin.evaluation_mode = evaluation_mode
	origin.evaluation_status = evaluation_status
	origin.creation_info = CreationInfo(agency_id=agency,
                                                    agency_uri=ResourceIdentifier(id=agency),
                                                    author="SeisMonitor",
                                                    author_uri=ResourceIdentifier(id="SeisMonitor"),
                                                    creation_time=UTCDateTime.now())
	return origin

def add_aditional_event_info(event,
				agency = None,
				event_type = "earthquake",
				event_type_certainty = "suspected"):
	
	ori_pref  = event.preferred_origin()
	if ori_pref == None:
		event.preferred_origin_id = event.origins[0].resource_id.id

	event.event_type = event_type
	event.event_type_certainty = event_type_certainty
	event.creation_info = CreationInfo(agency_id=agency,
										agency_uri=ResourceIdentifier(id=agency),
										author="SeisMonitor",
										author_uri=ResourceIdentifier(id="SeisMonitor"),
										creation_time=UTCDateTime.now())
	return event

def add_aditional_catalog_info(catalog,
				agency = None):
	catalog.creation_info = CreationInfo(agency_id=agency,
										agency_uri=ResourceIdentifier(id=agency),
										author="SeisMonitor",
										author_uri=ResourceIdentifier(id="SeisMonitor"),
										creation_time=UTCDateTime.now())
	return catalog


def preproc_stream(st,
				order=['normalize','merge','detrend',
						'taper',"filter"],
				decimate=None,detrend=None,filter=None,
				merge=None,normalize=None,
				resample=None,taper=None,
				select_networks=[], 
				select_stations=[],
				filter_networks=[], 
			  	filter_stations=[]):
	"""
	Parameters:
	-----------
	st: obspy.Stream object
		Stream object to preprocessing
	ppc_restrictions: PreprocRestrictions object
		Restrictions to preprocess a stream

	Returns:
	--------
	st: obspy.Stream object
		Preprocessed stream according to the order of the parameters. 
	processed: True or False
		True if was processed, False if not.
	"""
	tr = st[0]
	network = tr.stats.network
	station = tr.stats.station
	comment = ""

	if len(order) == 0:
		processed = False
		return st, processed, comment

	if (network in filter_networks) or\
		(station in filter_stations):
		processed = False

	else:
		if not select_networks:
			select_networks.append(network)
		if not select_stations:
			select_stations.append(station)
		
		if (network in select_networks) or\
		(station in select_stations):
			for i,process in enumerate(order):
				try:
					if process == 'decimate':
						st.decimate(**decimate)
					elif process == 'detrend':
						st.detrend(**detrend)
					elif process == 'filter':
						st.filter(**filter)
						# print("filter applied")
					elif process == 'merge':
						st.merge(**merge)
					elif process == 'normalize':
						st.normalize(**normalize)
					elif process == 'resample':
						st.resample(**resample)
					elif process == 'taper':
						st.taper(**taper)

					## only for print comments	
					if i == len(order)-1:
						comment += f"({process}:ok)"
					else:
						comment += f"({process}:ok)->"
				except:
					if i == len(order)-1:
						comment += f"({process}:Failed)"
					else:
						comment += f"({process}:Failed)->"
				processed = True
			comment = f"[{comment}]"
		else:
			processed = False

	return st, processed, comment

def get_csv_events(seiscomp_file, version="0.9", with_magnitude=True, 
					export = None, 
					from_format="SC3ML", inside_polygon=False ):
	"""
	parameters
	----------
	seiscom_file : str
		path of the sc3ml seiscomp file that contains only one event
	version: str
		String of the xml version number.
	with_magnitude: Bolean
		If True return the magnitude and magnitude information
	picker: None
		In 'eqt' or 'phasenet' doesn't matter the channel where is located the pick.
		While in others matters the channel. None is select to  have importance in the channel
	export: str (deault : None)
		Path to export in a csv file. None don't create any file, then only returns.
	returns
	-------
	appended_events : list
		It's a list of Pandas DataFrames where each dataframe contains all information about the picks from the respective event
	"""
	def change_xml_version(ev_file,new_version="0.9"):
		lines = open(ev_file).readlines()
		lines[1] = f'<seiscomp xmlns="http://geofon.gfz-potsdam.de/ns/seiscomp3-schema/{new_version}" version="{new_version}">\n'
		with open(ev_file, 'w') as f:
			f.write(''.join(lines))

	if from_format == "SC3ML":
		if version not in ["0.5", "0.6", "0.7", "0.8", "0.9", "0.10"]:
			change_xml_version(seiscomp_file,new_version="0.10")

	datefmt = "%Y-%m-%d %H:%M:%S.%f"
	seiscomp_file_path = os.path.splitext(seiscomp_file)[0]
	catalog = read_events(seiscomp_file,format = from_format)
	event_list = catalog.events 


	event_colname= ["n_event","event_id","event_time","latitude","latitude_uncertainty",
					"longitude","longitude_uncertainty","depth","depth_uncertainty",
					"rms","region","method","earth_model","event_type","magnitude",
					"magnitude_type","n_P_phases","n_S_phases"]
	pick_colname = ["n_event","event_id","pick_id","phasehint","arrival_time",
					"probability","snr","detection_probability",
					"network","station","location","channel","picker"
					]
	events_df = []
	picks_df = []
	for n_ev,event in enumerate(event_list):
		loc_id = os.path.basename(str(event.resource_id))
		ev_type = event.event_type
		# if event.creation_info != None:
		# 	agency = event.creation_info.agency_id
		# else:
		# 	agency = None

		if event.event_descriptions:
			region = event.event_descriptions[0].text
		else:
			region = None

		## Preferred Origin
		pref_origin = event.preferred_origin() 
		time = pref_origin.time
		latitude = pref_origin.latitude
		latitude_error = pref_origin.latitude_errors.uncertainty
		longitude = pref_origin.longitude
		longitude_error = pref_origin.longitude_errors.uncertainty

		depth = pref_origin.depth
		depth_error = pref_origin.depth_errors
		rms = pref_origin.quality.standard_error
		method = os.path.basename(str(pref_origin.method_id))
		earth_model = os.path.basename(str(pref_origin.earth_model_id))
		evaluation_mode = pref_origin.evaluation_mode

		if depth != None:
			depth = float(depth) #in km
		if depth_error != None:
			if depth_error.uncertainty != None:
				depth_error = depth_error.uncertainty #in km
			else:
				depth_error = None
		else:
			depth_error = None
		## Preferred Magnitude
		if with_magnitude:
			pref_magnitude = event.preferred_magnitude()
			if pref_magnitude != None:
				magnitude = pref_magnitude.mag
				magnitude_type = pref_magnitude.magnitude_type
				if magnitude != None:
					magnitude = round(magnitude,2)
			else:
				magnitude = None
				magnitude_type = None

		else:
			magnitude = None
			magnitude_type = None
		
		
		## Dictionary with the picks information
		picks = {}
		for pick in event.picks:
			if pick.resource_id.id not in picks.keys():
				
				if pick.creation_info == None:
					_author = None
				else:
					_author = pick.creation_info.author
				
				if pick.comments:
					comment = ast.literal_eval(pick.comments[0].text)
					prob = comment["probability"]
					if _author == "EQTransformer":
						snr = comment["snr"]
						ev_prob = comment["detection_probability"]
					else:
						snr = None
						ev_prob = None
				else:
					prob = None
					snr = None
					ev_prob = None


				

				picks[pick.resource_id.id] = {
											"id":os.path.basename(str(pick.resource_id)),
											"network_code":pick.waveform_id.network_code,
											"station_code":pick.waveform_id.station_code,
											"location_code":pick.waveform_id.location_code,
											"channel_code":pick.waveform_id.channel_code,
											"phase_hint":pick.phase_hint,
											"time":pick.time,
											"author":_author,
											"probability": prob,
											"snr":snr,
											"detection_probability":ev_prob,
											"time_errors":pick.time_errors,
											"filter_id":pick.filter_id,
											"method_id":pick.method_id,
											"polarity":pick.polarity,
											"evaluation_mode":pick.evaluation_mode,
											"evaluation_status":pick.evaluation_status } 
		
		p_count = 0
		s_count = 0
		for i,arrival in enumerate(pref_origin.arrivals):

			pick = picks[arrival.pick_id.id]
			pick_row = [n_ev,loc_id,pick["id"],pick["phase_hint"],
						pick["time"].datetime,pick["probability"],
						pick["snr"],pick["detection_probability"],
						pick["network_code"],pick["station_code"],
						pick["location_code"],pick["channel_code"],
						pick["author"]
						]
			
			picks_df.append(pick_row)

			if pick["phase_hint"].upper() == "P":
				p_count += 1
			elif pick["phase_hint"].upper() == "S":
				s_count += 1

		
		event_row = [n_ev,loc_id,time.datetime,latitude,latitude_error,\
					longitude,longitude_error,depth,depth_error,rms,region,\
					method, earth_model,ev_type,  magnitude, magnitude_type,
					p_count,s_count]

		events_df.append(event_row)

	events_df = pd.DataFrame(events_df,columns=event_colname)
	picks_df = pd.DataFrame(picks_df,columns=pick_colname)


	events_df = events_df.sort_values(by="event_time",
										ascending=True,
										ignore_index=True)
	picks_df = picks_df.sort_values(by="arrival_time",
										ascending=True,
										ignore_index=True)

	if export != None:
		if os.path.isdir(os.path.dirname(export)) == False:
			os.makedirs(os.path.dirname(export))
		events_df.to_csv(export,index=False)
		print(f"Events_csv_file: {export}")
	return events_df,picks_df



import os
from obspy.core.event.catalog import Catalog, read_events
from SeisMonitor.utils import printlog, isfile
import utils as ut

class Hypocenter():
    def __init__(self,catalog,xml_path,vel_path,out_dir):

        if isinstance(catalog,Catalog):
            self.catalog = catalog
        else:
            self.catalog = read_events(catalog)
        
        self.xml_path = xml_path
        self.vel_path = vel_path
        self.out_dir = out_dir
        self.out_file = os.path.join(self.out_dir,"hypocenter","hypocenter_events.xml")
        self.paths = os.path.join(self.out_dir,"hypocenter","hypocenter_paths")
        self.sta0 = os.path.join(self.paths,"STATION0.HYP")

    def locate(self,sfilename = "catalog.sfile",
                out_format="QUAKEML",
                rm_not_locatable=True):
        sfile = os.path.join(self.paths,sfilename)
        isfile(sfile)
        self.catalog.write(sfile,format="NORDIC")

        sta0 = ut.STATION0(self.xml_path,self.vel_path)
        sta0.write(self.sta0)

        hyp = ut.HypocenterTools(self.paths,sfilename)
        hyp.split()
        hyp.remodl_and_setbrn()
        hyp.update()

        ut.check_sfile_integrity(self.paths,rm_not_locatable)
        catalog = hyp.collect()
        if self.out_file != None:
            print ("Writing output file...")
            isfile(self.out_file)
            catalog.write(self.out_file, out_format)
        return catalog


from setuptools import setup, find_packages
import pathlib

import pkg_resources
import setuptools
import codecs
import os

# here = os.path.abspath(os.path.dirname(__file__))

# with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
#     long_description = "\n" + fh.read()

VERSION = '0.0.16'
DESCRIPTION = 'To monitor seismic activity'
LONG_DESCRIPTION = 'A package that allows to monitor the seismic activity through main steps in the monitoring workflow: earthquake detection and phase picking -> phase associator -> earthquake locator -> magnitude estimation.'

req_path = os.path.join(os.path.dirname(__file__),"requirements.txt")
with pathlib.Path('requirements.txt').open() as requirements_txt:
    install_requires = [
        str(requirement)
        for requirement
        in pkg_resources.parse_requirements(requirements_txt)
    ]

# Setting up
setup(
    name="SeisMonitor",
    version=VERSION,
    author="ecastillot (Emmanuel Castillo)",
    author_email="<ecastillot@unal.edu.co>",
    url="https://github.com/ecastillot/SeisMonitor",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=install_requires,
    keywords=['python', "seismonitor","earthquakes","seismology"],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
    ],
    python_requires='>=3.7.9'
)

# python setup.py sdist bdist_wheel
# twine upload dist/*